
/*****************************************************************************
 *                                    M L X                                  *
 *                 Rendering Library for Accelerated 3d Hardware             *
 *                           (C) SuSE GmbH 1997, 1998                        *
 *****************************************************************************/
/* author: simon pogarcic, sim@suse.de */



/*****************************************************************************/
/*****************************************************************************/
/* G200 OpenGL/Mesa driver                                                   */
/* ------------------------------------------------------------------------- */
/*****************************************************************************/
/*****************************************************************************/

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include <linux/pci.h>

#if 1
  #define LOG_DEBUG
#endif

#define LOG_DBGINFO "<g200>"
#include "log_debug.h"



#include "g200.h"



GENPROTO_DRIVER_INIT_MAIN(G200)



G200HardwareContext	g200_hwctx;
MGARenderingValues	g200_values;

static Tubyte 		g200_msg[] = "G200";
static Tubyte		g200_info[] = "G200 MLX Driver";

/******************************************************************************
 *                             G200 driver setup                              *
 ******************************************************************************/

MLXDriverRec G200_driver = {
	-1,
	CARD_G200,
	g200_msg, g200_info,
	G200Init,
	0, /* When you wrote these modules, uncomment this:
		(ACLMOD_RGBA | ACLMOD_LB | ACLMOD_ACCEL), */
    {
/* formatting of vertices */
	0,

/* renderer features */
	0,

/* depth test */
	ACLHW_DEPTH,

/* stencil test */
	0,

/* scissor test */
	0,

/* stippled primitives */
	0,

/* alpha buffer operations */
	0,

/* alpha blending operations */
	0,

/* tex operations */
    {
	0,
    },

/* color operations */
	0,

/* fog operations */
	0,

/* logic operations */
	0,

/* miscelaneous features */
	0,

/* number of independent texture units (for multitexturing) */
	1,

/* max alowed texture size in pixels */
	0,

/* max number of texture sets */
	0
    }
};



static Tint
G200_InitVidMem(ACLContext aclctx)
{
    MLX_INIT_ACCESS_CARD;

    Tubyte bytes_pp, pciaddrdiv;
    Tuint screen_size, free_pix_offset;
    ACLScreen scr = aclctx->Scr;
    Tint free_vidmem_size = CCTX(VidMemSize);
    Tbool fullscr = (aclctx->acl_cfg & ACLOPT_RUNMODE_FULL) ? 1 : 0;
    Tbool do_config = (!CCTX(lock) || (CCTX(lock) && fullscr)) ? 1 : 0;

    /* this info should be read from aclctx visual info... */
    Tbool do_config_back = 1;
    Tbool do_config_depth = 1;

LOG("G200_InitVidMem(): ######\n");

    if(fullscr) {
	if(CCTX(lock) && CCTX(is_full_screen)) {
	    ERR("###### Fullscreen already running! ######\n");
	    return 1;
	}
	CCTX(is_full_screen) = 1;
	sctx = CCTX(Screen) = &CCTX(FullScreen);
    }
    else {
	CCTX(is_full_screen) = 0;
	sctx = CCTX(Screen) = &CCTX(WinScreen);
    }

    if(do_config) {
	sctx->VisibleW = scr->VisibleWidth;
	sctx->VisibleH = scr->VisibleHeight;
	sctx->VirtualW = scr->VirtualWidth;
	sctx->VirtualH = scr->VirtualHeight;
	sctx->ScreenBpp = scr->ScreenBpp;

	/*
	 * A HACK: if you set screen_size == visible, all buffers could be
	 * seen, if you config card with virtual screen which covers whole
	 * video memory. The virtual width should be equal to visible width.
	 */
#if 1
	sctx->ScreenSize = sctx->VirtualW * sctx->VirtualH;
#else
	sctx->ScreenSize = sctx->VisibleW * sctx->VisibleH;
#endif

	/*
	 * bytes per pixel
	 */
	switch(sctx->ScreenBpp) {
	case 8:
	    sctx->BytesPP = 1;
	    break;
	case 15:
	case 16:
	    sctx->BytesPP = 2;
	    break;
	case 24:
	case 32:
	    sctx->BytesPP = 4;
	    break;
	default:
	    ERR("###### Unsupported ScreenBpp: %d ######\n", sctx->ScreenBpp);
	    return 1;
	}

#if 0
do you need this for g200 ?
	/*
	 * Partial Product values for screen width:
	 */
	sctx->ScreenPProd = ???
#endif

    }

    bytes_pp = sctx->BytesPP;



/* I don't beleive you need this for g200... */
    pciaddrdiv = (Tubyte)(8/bytes_pp);
/*********************************************/



    screen_size = sctx->ScreenSize;

    free_pix_offset = 0;

/*
 * free_vidmem_size counter is always in bytes. We assume that front, back and
 * depth buffer all require the same size which screen covers. The rest of mem
 * is taken for textures.
 */

/*##########################################################
This section should be somewhat rewritten for G200 driver...
 ###########################################################*/

/*****************************************************************************
 Front Buffer:
 *****************************************************************************/
    if(do_config) {
	if((free_vidmem_size -= (screen_size * bytes_pp)) < 0) {
	    ERR("###### no memory for framebuffer ######\n");
	    return 1;
	}
	/* pixels */
	sctx->FrontBuffOffset = free_pix_offset;
	/* quad words (64 bit) addressing */
	sctx->FrontBuffAddrOffset = free_pix_offset / pciaddrdiv;
    }
    free_pix_offset += screen_size;

/*****************************************************************************
 Back Buffer:
 *****************************************************************************/
    if(do_config || (!CCTX(win_back_set) && do_config_back)) {
	if((free_vidmem_size -= (screen_size * bytes_pp)) < 0) {
	    ERR("###### no memory for backbuffer ######\n");
	    return 1;
	}
	/* pixels */
	sctx->BackBuffOffset = free_pix_offset;
	sctx->FrontBackOffset = sctx->BackBuffOffset - sctx->FrontBuffOffset;
	/* quad words (64 bit) addressing */
	sctx->BackBuffAddrOffset = free_pix_offset / pciaddrdiv;
	CCTX(win_back_set) = 1;
    }
    if(CCTX(win_back_set))
	free_pix_offset += screen_size;

/*****************************************************************************
 Depth Buffer: depth buff always 16 bit (depth addr = 2 * pix address)
 *****************************************************************************/
    if(do_config || (!CCTX(win_depth_set) && do_config_depth)) {
	if((free_vidmem_size -= (screen_size * 2)) < 0) {
	    ERR("###### no memory for depthbuffer ######\n");
	    return 1;
	}
	/* bytes */
	sctx->DepthBuffOffset = free_pix_offset * 2;
	CCTX(win_depth_set) = 1;
    }
    if(CCTX(win_depth_set))
	free_pix_offset += (screen_size / 2);

/*****************************************************************************
 Textures Memory:
 *****************************************************************************/
    if(do_config) {
	if (free_vidmem_size <= 0) {
	    MSG("###### no memory for textures ######\n");
	    return 1;
	}
	/* pixels */
	sctx->TexBuffOffset = free_pix_offset;
	/* double words (32 bit) - the tex pixels are 32 bit addressed (?)*/
	sctx->TexBuffMin = (CCTX(VidMemSize) - free_vidmem_size) / 4;
	sctx->TexBuffMax = CCTX(VidMemSize) / 4;
    }
    
    return 0;
}



Tbool
G200Init(ACLContext aclctx)
{
    MLX_INIT_ACCESS_CARD;
    MLX_INIT_ACCESS_MMCTL;

    Tint hwidx = aclctx->hwidx;
    ACLScreen scr = aclctx->Scr;
    MLXDriverRec *drv = mlx_drivers[cctx->drvidx];
    GPMCardInfo_t *card = GpcliCardsInfo + hwidx;
    Tint dac, tmp = 0;
    
    MSG("\n");
    MSG("[%s] %s, Card: %d\n", drv->drvmsg, drv->drvinfo, hwidx);
    MSG("\n");

/***************** IMPORTANT!!! ***************/

    aclctx->CardCTL = aclctx->CardCTL0 = gpcli_mmap_pcidev(BASE_0, hwidx, 0);

    if ( !(MMIOBASE = aclctx->CardCTL) ) {
	ERR("###### CTL region not mapped! ######\n");
	return 1;
    }

    /*
     * set hardware/card context if not already
     ************************************************************************/
    if(!CCTX(lock)) {
#if 0
	GPMPciParam_t pciparam = {
	    hwidx, {
		{ PCI_COMMAND, SIZE_BYTE, 
		  PCI_COMMAND_IO | PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER,
		  0, FALSE },
		{ PCI_COMMAND, SIZE_BYTE, 
		  PCI_COMMAND_IO | PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER,
		  0, FALSE },
	    }
	};

	/*
	 * Before access, enable IO and memory of the card(s); we expect
	 * maximal two PCI devices on the card
	 *
	 * But it could happen that you loose your VTs if doing this...
	 ********************************************************************/
	gpcli_set_pciconfig( &pciparam );
#endif

	MSG("* SETTING NEW CARD CONTEXT...\n");

/* get RAMDAC info, optional... */
#if 0
	if(! (dac = GlintProbeRamdac(MMIOBASE))) {
	    ERR("###### RAMDAC not supported ######\n");
	    return 1;
	}
#endif

/* get RAMDAC info, optional... */
	CCTX(ExtDacType) = -1;
#if 0
	GlintFindRamdac(&(cctx->ExtDacType),dac);
#endif

/* set the whole size of vidmem here, REQUIRED */
#if 0
	tmp = REG_READ(PMMemConfig);
	CCTX(VidMemSize) = PMFBSize(tmp);
#endif

/* which VRAM ? Could be important... */
#if 0
	CCTX(MemType) = (REG_READ(PMRomControl) & 0x10) ? RAM_SDRAM: RAM_SGRAM;
	CCTX(is_vga) = (REG_READ(ChipConfig) & 0x2) ? 1 : 0;
#endif

/* enable/dissable pci retrys */
	/* take care with this stuff !!! */
#if 0
    #ifdef PCI_DISCONNECT
	REG_WRITE_SLOW(3, FIFODiscon);
    #else
	REG_WRITE_SLOW(0, FIFODiscon);
    #endif
#endif

    }

    if(G200_InitVidMem(aclctx)) {
	ERR("###### Cannot init VidMem!!! ######\n");
	return 1;
    }

    sctx = CCTX(Screen);

    if(CCTX(is_full_screen))
	VAL(win_x) = VAL(win_y) = 0;
    else
	/* one day... */
	VAL(win_x) = VAL(win_y) = 0;
	
    VAL(win_w) = scr->WindowWidth;
    VAL(win_h) = scr->WindowHeight;

    MSG("* CARD CONFIGURATION:\n");
    MSG(">>> G200 CTL mmaped to 0x%lx\n", (Tulong) aclctx->CardCTL);
    MSG(">>> FB (PCI Base 1) at 0x%lx; RAM: %dk\n",
	card->cd[0].PCIBase[1].base, CCTX(VidMemSize)/1024 );
    MSG(">>> RAMDAC type: ???\n"); 
    MSG(">>> Fitted Memory type: %s\n", "???");
    MSG(">>> VGA core: %s\n", cctx->is_vga ? "ENABLED" : "DISABLED");
    MSG("\n");
    MSG("* VIDEO MEMORY CONFIGURATION:\n");
    MSG(">>> Available Video RAM: %d Bytes\n", CCTX(VidMemSize));
    MSG(">>> Virtual Screen size: %d pix\n",sctx->VirtualW * sctx->VirtualH);
    MSG(">>> Visible Screen size: %d pix\n",sctx->VisibleW * sctx->VisibleH);
    MSG(">>> Used screen size:    %d pix\n", sctx->VisibleW * sctx->VisibleH);
    MSG(">>> Screen: pprod=0x%x, bpp=%d, bytespp=%d\n", sctx->ScreenPProd,
	sctx->ScreenBpp, sctx->BytesPP);
    MSG(">>> Front buffer: %d Kb, Offset: %d pixels\n",
	(sctx->ScreenSize * sctx->BytesPP)/1024, sctx->FrontBuffOffset);
    MSG(">>> Back buffer:  %d Kb, Offset %d pixels\n",
	(sctx->ScreenSize * sctx->BytesPP)/1024, sctx->BackBuffOffset);
    MSG(">>> Depth buffer: %d Kb, Offset %d words\n",
	(sctx->ScreenSize * 2)/1024, sctx->BackBuffOffset);
    MSG(">>> Tex memory:   %d Kb, Offset %d pixels\n",
	((sctx->TexBuffMax-sctx->TexBuffMin)*4)/1024, sctx->TexBuffOffset);
    MSG("\n");
    MSG("* SCREEN CONFIGURATION:\n");
    MSG(">>> Rendering into %s\n",CCTX(is_full_screen)?"FULLSCREEN":"WINDOW");
    MSG(">>> Virtual size (WxH): (%dx%d)\n",sctx->VirtualW, sctx->VirtualH);
    MSG(">>> Visible size (WxH): (%dx%d)\n",sctx->VisibleW, sctx->VisibleH);
    MSG(">>> Client X,Y: (%d,%d) size WxH: (%dx%d)\n",
	VAL(win_x), VAL(win_y), VAL(win_w), VAL(win_h));
    MSG("\n");

    return G200InitACLAPIModules(aclctx);
}

